/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Malte Kraus.
 *
 * Contributor(s):
 *  Malte Kraus <firefox@maltekraus.de> (Original author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function savePassword() {
  var user = document.getElementById("textbox-ftp-user").value;
  var password = document.getElementById("textbox-ftp-pass").value;
  var host = document.getElementById("textbox-ftp-host").value;
  if("nsILoginManager" in Ci) {
    var passwordManager = Cc["@mozilla.org/login-manager;1"]
                            .getService(Ci.nsILoginManager);
    var pseudoURI = "chrome://adblockfilters";

    // remove old logins:
    var logins = passwordManager.findLogins({}, pseudoURI, pseudoURI, null);
    for(var i = 0; i < logins.length; i++) {
      passwordManager.removeLogin(logins[i]);
    }

    if(password) { // add new login:
      var loginInfo = Cc["@mozilla.org/login-manager/loginInfo;1"]
                        .createInstance(Ci.nsILoginInfo);
      loginInfo.init(pseudoURI, pseudoURI, null, user, password, "", "");
      passwordManager.addLogin(loginInfo);
    }
  } else {
    var tokendb = Cc["@mozilla.org/security/pk11tokendb;1"]
                    .createInstance(Ci.nsIPK11TokenDB);
    var token = tokendb.getInternalKeyToken();

    if(!token.isLoggedIn()) {
      try {
        if(!token.checkPassword(""))
          token.login(true);
      } catch(e) {
        if(!token.isLoggedIn())
          return;
      }
    }

    var passwordManager = Cc["@mozilla.org/passwordmanager;1"]
                            .createInstance(Ci.nsIPasswordManager);
    try { passwordManager.removeUser(host, user); } catch(e) {}
    if(password)
      passwordManager.addUser(host, user, password);
  }
}

function initPassword() {
  var user = document.getElementById("textbox-ftp-user").value;
  var host = document.getElementById("textbox-ftp-host").value;
  if("nsILoginManager" in Ci) {
    var passwordManager = Cc["@mozilla.org/login-manager;1"]
                            .getService(Ci.nsILoginManager);
    // Find users for the given parameters
    var pseudoURI = "chrome://adblockfilters";
    var logins = passwordManager.findLogins({}, pseudoURI, pseudoURI, null);
    // Find user from returned array of nsILoginInfo objects
    for(var i = 0; i < logins.length; i++) {
      if(logins[i].username == user) {
        var textbox = document.getElementById("textbox-ftp-pass");
        textbox.value = textbox.defaultValue = logins[i].password;
        break;
      }
    }
  } else {
    // This doesn't harm if passwords are not encrypted
    var tokendb = Cc["@mozilla.org/security/pk11tokendb;1"]
                    .createInstance(Ci.nsIPK11TokenDB);
    var token = tokendb.getInternalKeyToken();

    if(!token.isLoggedIn()) {
      try {
        if(!token.checkPassword("")) {
          // So there's a master password. But since checkPassword didn't succeed,
          // we're logged out (per nsIPK11Token.idl).
          // Relogin and ask for the master password.
          token.login(true);  // 'true' means always prompt for token password.
                              // User will be prompted until clicking 'Cancel' or
                              // entering the correct password.

          // An exception will be thrown if the user cancels the login prompt
          // dialog. User is also logged out of Software Security Device.
        }
      } catch(e) {
        if(!token.isLoggedIn()) { // cancel
          //document.getElementById("textbox-ftp-pass").disabled = true;
          return;
        }
      }
    }

    var passwordManager = Cc["@mozilla.org/passwordmanager;1"]
                            .createInstance(Ci.nsIPasswordManager);
    var passwords = passwordManager.enumerator;
    while(passwords.hasMoreElements()) {
      var password = passwords.getNext();
      password.QueryInterface(Ci.nsIPassword);
      if(password.host == host && password.user == user) {
        var textbox = document.getElementById("textbox-ftp-pass");
        textbox.defaultValue = password.password;
        textbox.value = textbox.defaultValue;
        break;
      }
    }
  }
}

function initUploadPane() {
  initPassword();
  window.addEventListener("dialogaccept", savePassword, false);
}